; ----------------------------------------------------------------------------
; UsingApi.asm - 31 Mar 12 - twl
;
; This is a Win32 console program that writes to the console screen, gets user
; input, and then writes out the user's input to the screen, and then exits.
; It uses Win32 system calls from kernel32, and if using commandline assembly,
; it needs to be linked to kernel32.lib.
;
; This program was written using the C++ part of Visual Studio 2008 Pro. Once
; the programming environment is set, no special linking inputs are necessary. 
; If you are using VS, you can start with a General - Empty project. Right-clicking
; on the project name in the solution explorer, set Custom Build Rules so that the
; Microsoft Macro Assembler is used. Next, go to Project - Properties. Find the Linker
; in the dialog box and expand it (click on the +). Under the Linker, select System.
; On the top line, Subsystem, click the down arrow and choose "Console 
; (/SUBSYSTEM:CONSOLE)."In the same dialog box under Linker, select Advanced.
; Edit the top line, Entry Point, so that it is label that starts your code. In this
; example, it is _go. Note: You should not enter the underscore when filling in the 
; Entry Point. VS apparently adds an underscore.
; Your environment is now set to do assembly langauge programming
; 
; If do your assembly at the commandline -
; Assemble with "ml UsingApi.asm /c"
; Link with "link UsingApi kernel32.lib /subsystem:console /entry:go"
; ----------------------------------------------------------------------------

	        .386P
	        .model  flat
		    extern  _ExitProcess@4:near		;api call to exit program
			extern  _GetStdHandle@4:near	;api call to get std handle
			extern  _WriteConsoleA@20:near	;api call to write to the console
			extern	_ReadConsoleA@20:near	;api call to read from the console
	        public  _go						;the entry point for this program

	        .data							;start of the DATA segment
;
; constants
;	        
STD_INPUT_HANDLE	equ	-10					;api uses -10 to mean the input handle
STD_OUTPUT_HANDLE	equ	-11					;api uses -11 to mean the output handle
;
; memory storage
;
outHandle   DWORD 0							;stores the output handle
inHandle    DWORD 0							;stores the input handle
buffer      BYTE 80 dup(?)					;create an input buffer
bytesRead   DWORD ?							;holds number of bytes entered by the user
bytesWritten DWORD ?						;holds number of bytes written
;
; program messages
;
prgTitle    db 0ah, 0dh, "* * * Console IO Program * * *", 0ah, 0dh
pTitleLen	DWORD pTitleLen - prgTitle		;calculates length of message instead of
											;hard-coding a number in the main code
											
prompt      db 0ah, 0dh, "Enter some characters - "
promptLen	DWORD promptLen - prompt

youTyped    db 0ah, 0dh, "You typed...", 0ah, 0dh
youTypedLen	DWORD youTypedLen - youTyped

;------------------------------------------------------------------------------------
;
        .stack 4096							;give the program some stack space

;-------------------------------------------------------------------------------------
;
        .code								;start of the CODE segment
        								
_go     PROC								;entry point for the program at runtime
        ;
        ; get the handles for our input and output subroutines
        ;
        push    STD_OUTPUT_HANDLE			;tell the function what type of handle
        call    _GetStdHandle@4				;call the api function
        mov     outHandle, eax				;store the handle for our use

        push    STD_INPUT_HANDLE			;tell the function what type of handle
        call    _GetStdHandle@4				;call the api function
        mov     inHandle, eax				;store the handle for our use
		
		;
		; Write our program title to the console
		;
		mov	eax, pTitleLen					;get the number of characters in the message
		mov ebx, offset prgTitle			;get the address of the message
		call output							;call our print subroutine
        
        ;
        ; Write out our prompt to enter some characters
        ;
        mov eax, promptLen					;get the number of character in the message
        mov	ebx, offset prompt				;get the address of the message
        call output							;print it on the screen
               
        ;
        ; Get the user's input
        ;
        call input							;call the keyboard input subroutine
        
        ;
        ; print out the prompt
        ;
        mov eax, youTypedLen				;get the number of characters to print
        mov	ebx, offset youTyped			;get address of message to type
        call output							;print it on the console
        
		;
		; Write out what the user typed in
		;
		mov eax, bytesRead					;get the number of bytes to write
		mov	ebx, offset buffer				;point to the user's input string
		call output							;print it
 		
		;
        ; ExitProcess(0)
        ;
        push    0
        call    _ExitProcess@4

_go     endp

;
; * * * program subroutines * * *
;

;-------------------------------------------------------------------------------------
;output -	prints text messages to the console screen
;			Calling routing needs to put the number of characters to print into the
;			EAX register, and the address of the message in the EBX register. This
;			routine uses the Win32 API funciton _WriteConsoleA@20

output	PROC								

		;save the caller's regs
		push eax							;holds the number of characters to print
		push ebx							;holds the address of the message to print
		
		; WriteConsole(handle, &msg[0], 13, &written, 0) Parameters are pushed onto the stack
        ; in reverse order from how they are listed
        ;
        push    0							;last parameter first - reserved by MS
        push    offset bytesWritten			;tell function where to store the bytes actually written
        push    eax							;tell function how many chars to write
        push    ebx							;tell function where the message is to write
        push    outHandle					;tell function what handle to use
        call    _WriteConsoleA@20			;call the function
        
        ;restore the caller's regs
        pop eax
        pop ebx
        
        ret									;return to the caller
        
output	ENDP		

;-------------------------------------------------------------------------------------
;input -	Reads characters typed on the console keyboard. No registers need to be
;			passed. This routine uses the Win32 API funciton _ReadConsoleA@20. The
;			number of characters typed is stored in the bytesRead buffer
;

input	PROC								
		;
		; ReadConsole(handle, &inputbuffer, numberToRead, 0)
        ;
		; parameters are pushed on the stack in reverse order
		
        push    0							;last parameter first - reserved by MS
        push    offset bytesRead			;tell function where to store the bytes actually read
        push    80							;tell function max chars to read
        push    offset buffer				;tell function where to store user's input
        push    inHandle					;tell function what handle to use
        call    _ReadConsoleA@20			;call the function
        
        ret									;return to user
input	ENDP        
        
        END